                                                       (* 18. August 1998 *)

{
  NoS kann 0, 1, 2, 3, 4 oder 999 fuer jedes beliebige x annehmen
}

Unit WGleichu;

INTERFACE
USES WINCRT;                { nur zur Bildschirmausgabe der Rueckrechnung }

PROCEDURE GLEI1BIS4(Ax,Bx,Cx,Dx,Ex: DOUBLE; VAR X1r,X1i,X2r,X2i,X3r,X3i,
                    X4r,X4i: DOUBLE; VAR Gx: INTEGER);
PROCEDURE Ausgabe;

PROCEDURE Rueckrechnung;
PROCEDURE RAusgabe;

IMPLEMENTATION
{ -- private = (nur innerhalb der Unit globale) Konstanten u. Variablen --- }
CONST
EPS = 1.0E-10;                                       (* fuer Nullvergleich *)

TYPE COMPLEX = RECORD                                 (* zur Rueckrechnung *)
                 re : DOUBLE;
                 im : DOUBLE;
               END;

VAR
An,Bn,Cn,Dn,En : DOUBLE;                         (* Eingabekoeffizienten *)
X1re,X1im,X2re,X2im,X3re,X3im,X4re,X4im : DOUBLE;           (* Loesungen *)
Gl: INTEGER;             (* Welche Gleichung 4= 4ter Grad 3 = 3ter Grad .. *)
RAn,RBn,RCn,RDn,REn : DOUBLE;                  (* rueckgerechnete Koeff. *)
RA,RB,RC,RD : COMPLEX;

{ ------ private = (nur innerhalb der Unit globale) Unterprogramme -------- }
FUNCTION Sgn(An : DOUBLE) : INTEGER;
BEGIN
  IF(An < -EPS) THEN Sgn := -1
  ELSE BEGIN
    IF (An > EPS) THEN Sgn := 1
    ELSE  Sgn := 0;
  END;
END;

FUNCTION Arccos(X: DOUBLE) : DOUBLE;
BEGIN
    IF Sgn(X) = 0 THEN Arccos := Pi/2.0
    ELSE Arccos := Arctan(Sqrt(1-X*X)/X) + (1.0 - Sgn(X))*(Pi/2.0);
END;


PROCEDURE Quartic(An,Bn,Cn,Dn,En: DOUBLE; VAR X1re,X1im,X2re,X2im,
                  X3re,X3im,X4re,X4im: DOUBLE);

VAR

A,B,C,D : DOUBLE;       (* Koeffizienen der normierten Gleichung 4ten Gr.*)
                                  (* Koeff von x^4 ist 1                   *)
B1,C1,D1 : DOUBLE;        { Koeffizienten der reduzierten gleich 4ten Gr. }

P1re,P1im,P2re,P2im,P3re,P3im : DOUBLE;

T1re,T1im,T2re,T2im,T3re,T3im : DOUBLE;

FallRes : INTEGER;

(* ----------lokale Unterprogramme des Unterprogramms Quartic ------------ *)

PROCEDURE Standardform;
BEGIN
  IF Sgn(An)<> 0 THEN BEGIN
    A := Bn/An;
    B := Cn/An;
    C := Dn/An;
    D := En/An;
  END;
END;

PROCEDURE Reduktion;
BEGIN
  IF Sgn(A) = 0 THEN BEGIN
       B1 := B;
       C1 := C;
       D1 := D;
       END
  ELSE BEGIN
     B1 := B - (3.0*A*A)/8.0;
     C1 := C - (A*B)/2.0 + (A*A*A)/8.0;
     D1 := D - (A*C)/4.0 + (A*A*B)/16.0 - (3.0*A*A*A*A)/256.0;
  END;
{
  WRITELN('Koeffizienten der reduzierten Gleichung:  ');
  WRITELN;
  WRITELN('X^4  +       B1 * X^2   +         C1 * X   +          D1    =  0');
  WRITELN('       ',B1:8:4,'             ',C1:8:4,'             ',D1:8:4);
  READLN;
}
END;

PROCEDURE Resolvent;
                                  (* P^6 + A2*P^4 + B2*P^2 + C2 = 0 *)
VAR                               (* T^3 + A2*T^2 + B2*T +   C2 = 0 *)
  A2,B2,C2  : DOUBLE;
  Pk,Qk,Disk : DOUBLE;
  U,V,Uh,Vh,Phi : DOUBLE;
  Z1re,Z1im,Z2re,Z2im,Z3re,Z3im : DOUBLE;
                       (* Z = Loesung der reduzierten kubischen  Gleichung *)
                                           (*   Z^3 + Pk*Z + Qk = 0        *)
                       (*  Loesung der kubischen Resolvente  T = Z - A2/3  *)
  USign,VSign : INTEGER;

BEGIN
  A2 := 2.0*B1;
  B2 := B1*B1 - 4*D1;
  C2 := -C1*C1;

  Pk := -(A2*A2)/3.0 + B2;
                                                    (* Z^3 + Pk*Z + Qk = 0 *)
  Qk := 2.0*(A2*A2*A2)/27.0 - A2*B2/3.0 + C2;
                                                (*  Pk := -2.0; Qk := 1.0; *)
  Disk := Qk*Qk/4.0 + (Pk*Pk*Pk)/27.0;
  FallRes:= Sgn(Disk);
  CASE FallRes OF
                                       { Fall1:                             }
     1: BEGIN                          { 1 reelle,2 konj.komplexe Loesungen }
        Uh :=   -Qk/2.0 + SQRT(Disk);  USign := Sgn(Uh);
        Vh :=   -Qk/2.0 - SQRT(Disk);  VSign := Sgn(Vh);
        IF Sgn(Uh) = 0
          THEN U := 0.0
        ELSE U := Exp((1.0/3.0)*Ln(Abs(Uh))) * USign;
        IF Sgn(Vh) = 0
          Then V := 0.0
        ELSE V := Exp((1.0/3.0)*Ln(Abs(Vh))) * VSign;
        Z1re := U + V; Z1im := 0.0;
        Z2re := -(U + V)/2.0; Z2im := (U - V)/2.0 * Sqrt(3.0);
        Z3re := Z2re; Z3im := -Z2im;
        END;

     0: BEGIN                            { 1 reelle, 1 reelle Doppelloesung }
        Uh := -Qk/2.0; USign := Sgn(Uh);
        Vh := -Qk/2.0; VSign := Sgn(Vh);
        IF Sgn(Uh) = 0
          THEN U := 0.0
        ELSE U := Exp((1.0/3.0)*Ln(Abs(Uh))) * USign;
        IF Sgn(Vh) = 0
          Then V := 0.0
        ELSE V := Exp((1.0/3.0)*Ln(Abs(Vh))) * VSign;
        Z1re := 2*U; Z1im := 0.0;
        Z2re := -U; Z2im := 0.0;
        Z3re := Z2re; Z3im := Z2im;
        END;

    -1: BEGIN                                 { Fall3: casus irreducibilis }
        Uh := -(Pk*Pk*Pk)/27.0;
        Phi := Arccos((-Qk/2.0)/Sqrt(Uh));
        Z1re :=  2.0*Sqrt(-Pk/3.0) * Cos(Phi/3.0);
        Z2re :=  2.0*Sqrt(-Pk/3.0) * Cos(Phi/3.0 + (2.0*Pi/3.0));
        Z3re :=  2.0*Sqrt(-Pk/3.0) * Cos(Phi/3.0 - (2.0*Pi/3.0));
        Z1im := 0.0; Z2im := 0.0; Z3im := 0.0;
        END;
  END;
  T1re := Z1re - A2/3.0; T1im := Z1im;
  T2re := Z2re - A2/3.0; T2im := Z2im;
  T3re := Z3re - A2/3.0; T3im := Z3im;

{
  WRITELN('Koeffizienten der Resolvente:');
  WRITELN('T^3     +      A2 * T^2      +       B2 * T    +    C2  =  0');
  WRITELN('          ',A2:8:4,'              ',B2:8:4,'         ',C2:8:4);
  WRITELN;
  WRITELN;
  WRITELN;
  WRITELN('Koeffizienten der reduzierten Resolvente:');
  WRITELN('Z^3     +        Pk *Z           +        Qk       =  0');
  WRITELN('              ',Pk:8:4,'               ',Qk:8:4);
  WRITELN;
  WRITELN;
  WRITELN('Fallunterscheidung kubische Resolvente : ',FallRes);
  WRITELN;
  WRITELN('Loesungen der reduzierten kubischen Resolvente:');
  WRITELN('Z1re =  ',Z1re:8:4,'   Z1im =  ',Z1im:8:4);
  WRITELN('Z2re =  ',Z2re:8:4,'   Z2im =  ',Z2im:8:4);
  WRITELN('Z3re =  ',Z3re:8:4,'   Z3im =  ',Z3im:8:4);
  WRITELN;
  WRITELN('Loesungen der kubischen Resolvente:');
  WRITELN('T1re =  ',T1re:8:4,'   T1im =  ',T1im:8:4);
  WRITELN('T2re =  ',T2re:8:4,'   T2im =  ',T2im:8:4);
  WRITELN('T3re =  ',T3re:8:4,'   T3im =  ',T3im:8:4);
  WRITELN('Vorzeichen von C1: ',Sgn(C1));
  READLN;
}
END;

PROCEDURE Comsqrt2(A,B : DOUBLE;VAR U,V : DOUBLE);
VAR
Vz: DOUBLE;
BEGIN
  Vz := 1.0 + Sgn(B) - Sgn(B)*Sgn(B);
  U := Vz*Sqrt(0.5*(A + Sqrt(A*A + B*B)));
  V := Sqrt(0.5*(-A + Sqrt(A*A + B*B)));
END;

PROCEDURE P6Gleichung;
BEGIN
  Comsqrt2(T1re,T1im,P1re,P1im);
  Comsqrt2(T2re,T2im,P2re,P2im);
  Comsqrt2(T3re,T3im,P3re,P3im);
{
  WRITELN('Loesungen der Gleichung 6ten Grades:');
  WRITELN;
  WRITELN('P1re  = ',P1re:8:4,'   P1im = ',P1im:8:4);
  WRITELN('P2re  = ',P2re:8:4,'   P2im = ',P2im:8:4);
  WRITELN('P3re  = ',P3re:8:4,'   P3im = ',P3im:8:4);
  WRITELN;
  READLN;
}
END;

PROCEDURE Biquad(L:DOUBLE; M:DOUBLE);
VAR
  Ent : INTEGER;
  Disk,S,S1,S2,Sre,Sim,S1re,S1im,S2re,S2im : DOUBLE;
  Vz : DOUBLE;
BEGIN
  Disk := L*L - 4.0*M;
  Ent := Sgn(Disk);
  CASE Ent OF
       1 : BEGIN
           S1 := (-L + Sqrt(Disk)) / 2.0;
           S2 := (-L - Sqrt(Disk)) / 2.0;
           S := S1;
           X1re := 0.5*Sqrt(S*Sgn(S)) * (1.0 + Sgn(S));
           X1im := 0.5*Sqrt(S*Sgn(S)) * (1.0 - Sgn(S));
           X2re := -X1re;
           X2im := -X1im;
           S := S2;
           X3re := 0.5*Sqrt(S*Sgn(S)) * (1.0 + Sgn(S));
           X3im := 0.5*Sqrt(S*Sgn(S)) * (1.0 - Sgn(S));
           X4re := -X3re;
           X4im := -X3im;
           END;
       0 : BEGIN
           S1 := -L / 2.0;
           S2 := S1;
           S := S1;
           X1re := 0.5*Sqrt(S*Sgn(S)) * (1.0 + Sgn(S));
           X1im := 0.5*Sqrt(S*Sgn(S)) * (1.0 - Sgn(S));
           X2re := -X1re;
           X2im := -X1im;
           X3re := X1re;
           X3im := X1im;
           X4re := X2re;
           X4im := X2im;
           END;
      -1 : BEGIN
           S1re := -L / 2.0;
           S1im := Sqrt(-Disk) / 2.0;
           S2re := -L / 2.0;
           S2im := -Sqrt(-Disk) / 2.0;
           Sre := S1re;
           Sim := S1im;
         {  WRITE('S1re =  ',S1re:8:4); WRITELN('   S1im = ',S1im:8:4); }
           X1re := Sqrt(0.5*(Sre + Sqrt(Sre*Sre + Sim*Sim)));
           Vz := 1.0 + Sgn(Sim) - Sgn(Sim) * Sgn(Sim);
           X1im := Vz*Sqrt(0.5*(-Sre + Sqrt(Sre*Sre + Sim*Sim)));
           X2re := -X1re;
           X2im := -X1im;
           Sre := S2re;
           Sim := S2im;
         {  WRITE('S2re =  ',S2re:8:4); WRITELN('   S2im = ',S2im:8:4); }
           X3re := -Sqrt(0.5*(Sre + Sqrt(Sre*Sre + Sim*Sim)));
           Vz := 1.0 + Sgn(Sim) - Sgn(Sim) * Sgn(Sim);
           X3im := -Vz*Sqrt(0.5*(-Sre + Sqrt(Sre*Sre + Sim*Sim)));
           X4re := -X3re;
           X4im := -X3im;
           END;
  END;
  IF (Sgn(c1) = 0) THEN BEGIN
    X1re := X1re - A/4.0;
    X2re := X2re - A/4.0;
    X3re := X3re - A/4.0;
    X4re := X4re - A/4.0;
  {  WRITELN('reduzierte Gleichung ist biquadratisch'); }
  END;
END;

PROCEDURE kombiplus;
BEGIN
  X1re := 0.5 * (P1re + P2re + P3re) - A/4.0;
  X2re := 0.5 * (P1re - P2re - P3re) - A/4.0;
  X3re := 0.5 * (-P1re - P2re + P3re) - A/4.0;
  X4re := 0.5 * (-P1re + P2re - P3re) - A/4.0;

  X1im := 0.5 * (P1im + P2im + P3im);
  X2im := 0.5 * (P1im - P2im - P3im);
  X3im := 0.5 * (-P1im - P2im + P3im);
  X4im := 0.5 * (-P1im + P2im - P3im);
{  WRITELN('Es wurde kombiplus ausgefuehrt'); }
END;

PROCEDURE kombiminus;
BEGIN
  X1re := 0.5 * (-P1re - P2re - P3re) - A/4.0;
  X2re := 0.5 * (-P1re + P2re + P3re) - A/4.0;
  X3re := 0.5 * (P1re + P2re - P3re) - A/4.0;
  X4re := 0.5 * (P1re - P2re + P3re) - A/4.0;

  X1im := 0.5 * (-P1im - P2im - P3im);
  X2im := 0.5 * (-P1im + P2im + P3im);
  X3im := 0.5 * (P1im + P2im - P3im);
  X4im := 0.5 * (P1im - P2im + P3im);
{  WRITELN('Es wurde kombiminus ausgefuehrt'); }
END;

PROCEDURE kombination;
VAR
Fall,vzt : INTEGER;
BEGIN
 vzt := Sgn(t1re) + Sgn(t2re) + Sgn(t3re);
  IF ((Fallres = -1) OR (Fallres = 0)) THEN BEGIN
    Fall := vzt + Sgn(c1);
   { WRITELN('FALL =',Fall:3); }
    CASE Fall OF
      4: kombiminus;
      2: kombiplus;
      0: kombiplus;
     -2: kombiminus;
    END;
  END;

  IF (Fallres = 1) THEN BEGIN
    IF (Sgn(c1) = 1) THEN kombiplus
    ELSE kombiminus;
  END;
END;

BEGIN                                                       (* von Quartic *)
  Standardform;
  IF ((Sgn(A) = 0) AND (Sgn(C) = 0)) THEN BEGIN
    { WRITELN('biquad schon bei der Eingabe');
     READLN;   }
     biquad(B,D);
  END
  ELSE BEGIN
    reduktion;
    IF (Sgn(C1) = 0) THEN BEGIN
      biquad(B1,D1);
    END
    else BEGIN
      resolvent;
      p6gleichung;
      kombination;
    END;
  END;
END;                                                        (* von Quartic *)

PROCEDURE Cubic;

VAR                                      (* X^3 + A2*X^2 + B2*X +   C2 = 0 *)
  A2,B2,C2  : DOUBLE;
  FallKub : INTEGER;
  Pk,Qk,Disk : DOUBLE;
  U,V,Uh,Vh,Phi : DOUBLE;
  Z1re,Z1im,Z2re,Z2im,Z3re,Z3im : DOUBLE;
                       (* Z = Loesung der reduzierten kubischen  Gleichung *)
                                           (*   Z^3 + Pk*Z + Qk = 0        *)
                       (*  Loesung der kubischen Gleichung   X = Z - A2/3  *)
  USign,VSign : INTEGER;

BEGIN
  A2 := Cn/Bn;
  B2 := Dn/Bn;
  C2 := En/Bn;

  Pk := -(A2*A2)/3.0 + B2;
                                                    (* Z^3 + Pk*Z + Qk = 0 *)
  Qk := 2.0*(A2*A2*A2)/27.0 - A2*B2/3.0 + C2;
                                                (*  Pk := -2.0; Qk := 1.0; *)
  Disk := Qk*Qk/4.0 + (Pk*Pk*Pk)/27.0;
  FallKub := Sgn(Disk);
  CASE FallKub OF
                                       { Fall1:                             }
     1: BEGIN                          { 1 reelle,2 konj.komplexe Loesungen }
        Uh :=   -Qk/2.0 + SQRT(Disk);  USign := Sgn(Uh);
        Vh :=   -Qk/2.0 - SQRT(Disk);  VSign := Sgn(Vh);
        IF Sgn(Uh) = 0
          THEN U := 0.0
        ELSE U := Exp((1.0/3.0)*Ln(Abs(Uh))) * USign;
        IF Sgn(Vh) = 0
          Then V := 0.0
        ELSE V := Exp((1.0/3.0)*Ln(Abs(Vh))) * VSign;
        Z1re := U + V; Z1im := 0.0;
        Z2re := -(U + V)/2.0; Z2im := (U - V)/2.0 * Sqrt(3.0);
        Z3re := Z2re; Z3im := -Z2im;
        END;

     0: BEGIN                            { 1 reelle, 1 reelle Doppelloesung }
        Uh := -Qk/2.0; USign := Sgn(Uh);
        Vh := -Qk/2.0; VSign := Sgn(Vh);
        IF Sgn(Uh) = 0
          THEN U := 0.0
        ELSE U := Exp((1.0/3.0)*Ln(Abs(Uh))) * USign;
        IF Sgn(Vh) = 0
          Then V := 0.0
        ELSE V := Exp((1.0/3.0)*Ln(Abs(Vh))) * VSign;
        Z1re := 2*U; Z1im := 0.0;
        Z2re := -U; Z2im := 0.0;
        Z3re := Z2re; Z3im := Z2im;
        END;

    -1: BEGIN                                 { Fall3: casus irreducibilis }
        Uh := -(Pk*Pk*Pk)/27.0;
        Phi := Arccos((-Qk/2.0)/Sqrt(Uh));
        Z1re :=  2.0*Sqrt(-Pk/3.0) * Cos(Phi/3.0);
        Z2re :=  2.0*Sqrt(-Pk/3.0) * Cos(Phi/3.0 + (2.0*Pi/3.0));
        Z3re :=  2.0*Sqrt(-Pk/3.0) * Cos(Phi/3.0 - (2.0*Pi/3.0));
        Z1im := 0.0; Z2im := 0.0; Z3im := 0.0;
        END;
  END;
  X1re := Z1re - A2/3.0; X1im := Z1im;
  X2re := Z2re - A2/3.0; X2im := Z2im;
  X3re := Z3re - A2/3.0; X3im := Z3im;
{
  CLRSCR;
  WRITELN;
  WRITELN('Koeffizienten der reduzierten Gleichung:');
  WRITELN('Z^3     +        Pk *Z           +        Qk       =  0');
  WRITELN('              ',Pk:8:4,'               ',Qk:8:4);
  WRITELN;
  WRITELN;
  WRITELN('Fallunterscheidung kubische Gleichung : ',FallKub);
  WRITELN;
  WRITELN('Loesungen der reduzierten kubischen Gleichung:');
  WRITELN('Z1re =  ',Z1re:8:4,'   Z1im =  ',Z1im:8:4);
  WRITELN('Z2re =  ',Z2re:8:4,'   Z2im =  ',Z2im:8:4);
  WRITELN('Z3re =  ',Z3re:8:4,'   Z3im =  ',Z3im:8:4);
  READLN;
}
END;

PROCEDURE Quadric;
VAR
P,Q,Diskr : DOUBLE;
BEGIN
  P := Dn/Cn; Q := En/Cn;
  Diskr := -P/2.0 * (-P/2.0) - Q;
  IF Sgn(Diskr) > 0 THEN BEGIN
     X1re := -P/2.0 + SQRT(Diskr); X1im := 0.0;
     X2re := -P/2.0 - SQRT(Diskr);  X2im := 0.0;
     END ELSE IF Sgn(Diskr) = 0 THEN BEGIN
        X1re := -P/2.0; X1im := 0.0;
        X2re := X1re; X2im := 0.0;
     END ELSE BEGIN
       X1re := -P/2.0;
       X1im := SQRT(-Diskr);
       X2re := X1re;
       X2im := -SQRT(-Diskr);
     END;
END;

PROCEDURE Linear;
VAR
A,B:DOUBLE;
BEGIN
  A := Dn;
  B := En;
  X1re := -B/A;
END;

PROCEDURE Allx;
BEGIN
END;

PROCEDURE NoEqu;
BEGIN
END;


PROCEDURE WelcheGleichung(VAR Gl : INTEGER);
BEGIN
  IF Sgn(An) <> 0 THEN Gl := 4
  ELSE IF Sgn(Bn) <> 0 THEN Gl := 3
    ELSE IF Sgn(Cn) <> 0 THEN Gl := 2
      ELSE IF Sgn(Dn) <> 0 THEN Gl:= 1
        ELSE If Sgn(En) <> 0 THEN GL := 0
        ELSE Gl := 999;
END;

PROCEDURE Loesung;
BEGIN
  CASE Gl OF
    4: Quartic(An,Bn,Cn,Dn,En,X1re,X1im,X2re,X2im,X3re,X3im,X4re,X4im);
    3: Cubic;
    2: Quadric;
    1: Linear;
    0: NoEqu;
  999: Allx;
  END;
END;

PROCEDURE Ausgabe;
BEGIN
  CLRSCR;
  CASE Gl OF
  4: BEGIN
       WRITELN;
       WRITELN;
       WRITELN('Loesungen der Gleichung vierten Grades:');
       WRITELN;
       WRITE('        A * X^4   +  B * X^3    +   C * X^2    +   D * X  + ');
       WRITELN('  E    =   0 ');
       WRITELN;
       WRITE(An:10:2,'   ',Bn:10:2,'     ',Cn:10:2,'     ',Dn:10:2,' ');
       WRITELN(En:10:2,'   =   0');
       WRITELN;
       WRITELN;
       WRITELN('      Real          Imag');
       WRITELN('X1: ',X1re:8:4,'      ',X1im:8:4);
       WRITELN('X2: ',X2re:8:4,'      ',X2im:8:4);
       WRITELN('X3: ',X3re:8:4,'      ',X3im:8:4);
       WRITELN('X4: ',X4re:8:4,'      ',X4im:8:4);
     END;
  3: BEGIN
       WRITELN;
       WRITELN;
       WRITELN('Loesungen der Gleichung dritten Grades:');
       WRITELN;
       WRITE('       A * X^3    +   B * X^2    +   C * X  +   D ');
       WRITELN('     =   0');
       WRITELN;
       WRITE(Bn:10:2,'     ',Cn:10:2,'     ',Dn:10:2,' ',En:10:2);
       WRITELN('    =   0');
       WRITELN;
       WRITELN;
       WRITELN;
       WRITELN('      Real          Imag');
       WRITELN('X1: ',X1re:8:4,'      ',X1im:8:4);
       WRITELN('X2: ',X2re:8:4,'      ',X2im:8:4);
       WRITELN('X3: ',X3re:8:4,'      ',X3im:8:4);
     END;
  2: BEGIN
       WRITELN;
       WRITELN;
       WRITELN;
       WRITELN('Loesungen der  quadratischen Gleichung:');
       WRITELN;
       WRITE('       A * X^2    +   B * X  +   C');
       WRITELN('      =   0');
       WRITELN;
       WRITE(Cn:10:2,'     ',Dn:10:2,' ',En:10:2);
       WRITELN('    =   0');
       WRITELN;
       WRITELN;
       WRITELN;
       WRITELN('      Real          Imag');
       WRITELN('X1: ',X1re:8:4,'      ',X1im:8:4);
       WRITELN('X2: ',X2re:8:4,'      ',X2im:8:4);
     END;
  1: BEGIN
       WRITELN;
       WRITELN;
       WRITELN('Loesung der linearen Gleichung:');
       WRITELN;
       WRITE('       A * X  +   B');
       WRITELN('    =   0');
       WRITELN;
       WRITE(Dn:10:2,' ',En:10:2);
       WRITELN('    =   0');
       WRITELN;
       WRITELN('X1: ',X1re:8:4);
     END;
999: BEGIN
       WRITELN;
       WRITELN('eingegebene Gleichung:');
       WRITELN;
       WRITE('       A * X  +   B');
       WRITELN('      =    0');
       WRITELN;
       WRITE(Dn:10:2,' ',En:10:2);
       WRITELN('    =    0');
       WRITELN;
       WRITELN;
       WRITELN('Die Gleichung ist fuer jedes X erfuellt');
     END;
  0: BEGIN
       WRITELN;
       WRITELN('eingegebene Gleichung:');
       WRITELN;
       WRITE('       A * X  +   B');
       WRITELN('      =    0');
       WRITELN;
       WRITE(Dn:10:2,' ',En:10:2);
       WRITELN('    =    0');
       WRITELN;
       WRITELN;
       WRITELN('Es gibt keine Loesung');
     END;
  END;
END;

(* zur Rueckrechnung benoetigte Prozeduren *)

PROCEDURE Ca(VAR ZA : COMPLEX;ZB : COMPLEX);
  BEGIN
  ZA.re := ZA.re + ZB.re;
  ZA.im := ZA.im + ZB.im;
  END;

PROCEDURE Cm(VAR ZA : COMPLEX;ZB : COMPLEX);
  VAR
  H : COMPLEX;
  BEGIN
  H.re := ZA.re;
  H.im := ZA.im;
  ZA.re := (ZA.re*ZB.re)- (ZA.im*ZB.im);
  ZA.im := (H.re*ZB.im) + (H.im *ZB.re);
  END;

PROCEDURE Cneg(VAR ZA : COMPLEX);
  BEGIN
  ZA.re := -ZA.re;
  ZA.im := -ZA.im;
  END;


PROCEDURE RQuartic(X1re,X1im,X2re,X2im,X3re,X3im,X4re,X4im: DOUBLE; VAR
                   RA,RB,RC,RD: COMPLEX; VAR RAn,RBn,RCn,RDn,REn : DOUBLE);
  VAR
  H1,H2,H3,H4,H5,H6: COMPLEX;
  BEGIN
  H1.re := X1re; H1.im := X1im;  H2.re := X2re; H2.im := X2im;
  H3.re := X3re; H3.im := X3im;  H4.re := X4re; H4.im := X4im;

  Ca(H1,H2);
  Ca(H3,H4);
  Ca(H1,H3);
  Cneg(H1);
  RA := H1;                                               { negative Summe }

  H1.re := X1re; H1.im := X1im; H3.re := X3re; H3.im := X3im;
  Cm(H1,H2); H5.re := X1re; H5.im := X1im;
  Cm(H5,H3);
  Ca(H1,H5);
  RB := H1;

  H1.re := X1re; H1.im := X1im;
  Cm(H1,H4);
  Cm(H2,H3);
  Ca(H1,H2);
  Ca(RB,H1);

  H2.re := X2re; H2.im := X2im; H3.re := X3re; H3.im := X3im;
  Cm(H2,H4);
  Cm(H3,H4);
  Ca(H2,H3);
  Ca(RB,H2);

  H1.re := X1re; H1.im := X1im;
  H2.re := X2re; H2.im := X2im; H3.re := X3re; H3.im := X3im;
  Cm(H1,H2);  H5 := H1;
  Cm(H1,H3);
  Cm(H5,H4);
  Ca(H1,H5);
  RC := H1;
  H1.re := X1re; H1.im := X1im;
  Cm(H1,H3);
  Cm(H1,H4);
  Cm(H2,H3);
  Cm(H2,H4);
  Ca(H1,H2);
  Ca(RC,H1);
  Cneg(RC);

  H1.re := X1re; H1.im := X1im;  H2.re := X2re; H2.im := X2im;
  H3.re := X3re; H3.im := X3im;  H4.re := X4re; H4.im := X4im;

  Cm(H1,H2);
  Cm(H3,H4);
  Cm(H1,H3);
  RD := H1;

  RAn := An;
  RBn := RA.re * An;
  RCn := RB.re * An;
  RDn := RC.re * An;
  REn := RD.re * An;
  END;

PROCEDURE RCubic(X1re,X1im,X2re,X2im,X3re,X3im: DOUBLE; VAR RA,RB,RC:
                 COMPLEX; VAR RBn,RCn,RDn,REn : DOUBLE);
  VAR
  H1,H2,H3,H4: COMPLEX;
  BEGIN
  H1.re := X1re; H1.im := X1im;  H2.re := X2re; H2.im := X2im;
  H3.re := X3re; H3.im := X3im;
  Ca(H1,H2);
  Ca(H1,H3);
  Cneg(H1);
  RA := H1;

  H1.re := X1re; H1.im := X1im; H4 := H1;
  Cm(H1,H2); H4.re := X1re; H4.im := X1im;
  Cm(H4,H3);
  Cm(H2,H3);
  Ca(H1,H4);
  Ca(H1,H2);
  RB := H1;

  H1.re := X1re; H1.im := X1im;  H2.re := X2re; H2.im := X2im;
  Cm(H1,H2);
  Cm(H1,H3);
  Cneg(H1);
  RC := H1;

  RBn := Bn;
  RCn := RA.re * Bn;
  RDn := RB.re * Bn;
  REn := RC.re * Bn;
  END;

PROCEDURE RQuadric(X1re,X1im,X2re,X2im: DOUBLE; VAR RA,RB: COMPLEX;
                   VAR RCn,RDn,REn: DOUBLE);
  VAR
  H1,H2 : COMPLEX;
  BEGIN
    H1.re := X1re; H1.im := X1im;  H2.re := X2re; H2.im := X2im;
    Ca(H1,H2);
    Cneg(H1);
    RA := H1;

    H1.re := X1re; H1.im := X1im;
    Cm(H1,H2);
    RB := H1;

    RCn := Cn;
    RDn := RA.re * Cn;
    REn := RB.re * Cn;
  END;

PROCEDURE RLinear(X1re,X1im: DOUBLE; VAR RA: COMPLEX;
                   VAR RDn,REn: DOUBLE);
  VAR
  H1 : COMPLEX;
  BEGIN
    H1.re := X1re; H1.im := X1im;
    RDn := Dn;
    REn := -X1re*Dn;
  END;

PROCEDURE RAllx;
BEGIN
  { Keine Rueckrechnung zielfuehrend }
END;

PROCEDURE RNoEqu;
  BEGIN
    { Keine Rueckrechnung zielfuehrend }
  END;

PROCEDURE Rueckrechnung;
BEGIN
  CASE Gl OF
    4: RQuartic(X1re,X1im,X2re,X2im,X3re,X3im,X4re,X4im,RA,RB,RC,RD,
                RAn,RBn,RCn,RDn,REn);
    3: RCubic(X1re,X1im,X2re,X2im,X3re,X3im,RA,RB,RC,RBn,RCn,RDn,REn);
    2: RQuadric(X1re,X1im,X2re,X2im,RA,RB,RCn,RDn,REn);
    1: RLinear(X1re,X1im,RA,RDn,REn);
    0: RNoEqu;
  999: RAllx;
  END;
END;

PROCEDURE RAusgabe;
  BEGIN
  CLRSCR;
  CASE Gl OF
  4: BEGIN
       WRITELN;
       WRITELN;
       WRITELN('Rueckrechnung der Gleichung vierten Grades:');
       WRITELN;
       WRITE('        A * X^4   +  B * X^3    +   C * X^2    +   D * X  + ');
       WRITELN('  E    =   0 ');
       WRITELN;
       WRITE(RAn:10:2,'   ',RBn:10:2,'     ',RCn:10:2,'     ',RDn:10:2,' ');
       WRITELN(REn:10:2,'   =   0');
     END;
  3: BEGIN
       WRITELN;
       WRITELN;
       WRITELN('Rueckrechnung der Gleichung dritten Grades:');
       WRITELN;
       WRITE('       A * X^3    +   B * X^2    +   C * X  +   D ');
       WRITELN('     =   0');
       WRITELN;
       WRITE(RBn:10:2,'     ',  RCn:10:2,'     ',RDn:10:2,' ',REn:10:2);
       WRITELN('    =   0');
     END;
  2: BEGIN
       WRITELN;
       WRITELN;
       WRITELN;
       WRITELN('Rueckrechnung der  quadratischen Gleichung:');
       WRITELN;
       WRITE('       A * X^2    +   B * X  +   C');
       WRITELN('      =   0');
       WRITELN;
       WRITE(RCn:10:2,'     ',RDn:10:2,' ',REn:10:2);
       WRITELN('    =   0');
     END;
  1: BEGIN
       WRITELN;
       WRITELN;
       WRITELN('Rueckrechnung der linearen Gleichung:');
       WRITELN;
       WRITE('       A * X  +   B');
       WRITELN('    =   0');
       WRITELN;
       WRITE(RDn:10:2,' ',REn:10:2);
       WRITELN('    =   0');
     END;
999: BEGIN
       WRITELN;
       WRITELN('Rueckrechnung der eingegebenen Gleichung:');
       WRITELN;
       WRITE('       A * X  +   B');
       WRITELN('      =    0');
       WRITELN;
       WRITE(Dn:10:2,' ',En:10:2);
       WRITELN('    =    0');
       WRITELN;
       WRITELN;
       WRITE('Rueckrechnung nicht zielfuehrend:');
       WRITELN(' da die Gleichung fuer jedes X erfuellt ist.');
     END;
  0: BEGIN
       WRITELN;
       WRITELN('Rueckrechnung der eingegebenen Gleichung:');
       WRITELN;
       WRITE('       A * X  +   B');
       WRITELN('      =    0');
       WRITELN;
       WRITE(Dn:10:2,' ',En:10:2);
       WRITELN('    =    0');
       WRITELN;
       WRITELN;
       WRITE('Rueckrechnung nicht zielfuehrend:');
       WRITELN(' da es keine Loesung gibt');
     END;
  END;
END;


PROCEDURE GLEI1BIS4(Ax,Bx,Cx,Dx,Ex: DOUBLE; VAR X1r,X1i,X2r,X2i,X3r,X3i,
                    X4r,X4i: DOUBLE; VAR Gx: INTEGER);
BEGIN
  An := Ax; Bn := Bx; Cn := Cx; Dn := Dx; En := Ex;
  Gl := Gx;
  WelcheGleichung(Gl); Gx := Gl;
  Loesung;
  X1r := X1re; X1i := X1im;   X2r := X2re; X2i := X2im;
  X3r := X3re; X3i := X3im;   X4r := X4re; X4i := X4im;
END;

BEGIN                                              { Keine Initialisierung }
END.