// quadrglc                                             Jaenner 1999
/*
  Genauigkeit beim Maschinellen Gleichungsloesen an Hand der quadratischen
  Gleichung. x^2 + px + q = 0

  erste Testphase:
  p: -10^3   bis  -10^12           q: 1      ausprobieren.

  zweite Testphase:
  p: -10^100  bis -10^300          q: 10000      ausprobieren.
  eventuell das Ausgabeformat von  #LE auf Lf aendern
*/

#include <stdio.h>
#include <conio.h>
#include <math.h>

#define EPS 1.0e-12                                 /* fuer Nullvergleich */

struct komplex                                // Record fr Komplexe Zahlen
 {
    long double re;
    long double im;
 };

long double p,q;                                    // Eingabekoeffizienten
long double Hrpre,Hrqre,Mrpre,Mrqre;                // r = zurueckgerechnet
long double Hrpim,Hrqim,Mrpim,Mrqim;
long double Hx1re,Hx2re,Mx1re,Mx2re;            // H = haendisch M = Maschine
long double Hx1im,Hx2im,Mx1im,Mx2im;

/* --------------------- Prototypen der Unterprogramme ------------------ */
// Nur bei den Prototypen muss man void in der Parameterliste schreiben !!!
// Bei den eigentlichen Definitionen nur () !!!

int sgn(long double);
long double arccos(long double);
struct komplex kadd(struct komplex z1, struct komplex z2);
struct komplex kmult(struct komplex z1, struct komplex z2);
struct komplex kneg(struct komplex z);
void rueck2(void);
void RHaendisch(void);
void RMaschine(void);
void Eingabe(void);
void Ueberschrift(void);
void Haendisch(void);
void Maschine(void);
void RUeberschrift(void);

/* ------------------------ Hauptprogramm ------------------------------ */
void main()
{
  clrscr();
  Eingabe();
  Ueberschrift();
  Haendisch();       // bei sehr grossem p mit Kommentarklammern ausklammern
  Maschine();
  getch();
  clrscr();
  RUeberschrift();
  RHaendisch();
  RMaschine();
  getch();
}


/* ------------------------ Unterprogramme ------------------------------ */
int sgn(long double x) {
  int vz;
  if(x < -EPS) vz = -1;
  else {
    if (x > EPS) vz = 1;
      else vz = 0;
  }
  return(vz);
}

long double  arccos(long double x) {
  long double u;
  if (sgn(x) == 0)  return(M_PI/2.0);
  else u = atan(sqrtl(1-x*x)/x) + (1.0 - sgn(x))*(M_PI/2.0);
  return(u);
}

void Eingabe()
{
clrscr();

  printf("Das Programm loest die quadratische Gleichung von der Form\n");
  printf(" x^2 + p*x + q = 0  auf zwei Arten:\n");
  printf(" fuer p =  -10^3 bis -10^12         fuer q = 1 eingeben\n\n");
  printf("p (z.B.: -1.0E10) : ");
  scanf("%Lf",&p);
  printf("\nq (z.B.: 1.0) : ");
  scanf("%Lf",&q);
}


void Ueberschrift()
{
  printf("\n\nLoesungen der  quadratischen Gleichung:\n");
  printf("X^2    +   p          *  x      +           q");
  printf("      = 0\n");
  printf("x^2    % #LE  *  x           ",p);
  printf("% #LE = 0\n\n\n",q);
}

void Haendisch()
{
  long double Diskr;

  Diskr = -p/2.0 * (-p/2.0) - q;
  if (sgn(Diskr) > 0) {
     Hx1re = -p/2.0 + sqrtl(Diskr); Hx1im = 0.0;
     Hx2re = -p/2.0 - sqrtl(Diskr); Hx2im = 0.0;
     } else if (sgn(Diskr) == 0) {
	Hx1re = -p/2.0; Hx1im = 0.0;
	Hx2re = Hx1re; Hx2im = 0.0;
     } else {
       Hx1re = -p/2.0;
       Hx1im = sqrtl(-Diskr);
       Hx2re = Hx1re;
       Hx2im = -sqrtl(-Diskr);
     }

  printf("Mit der ueblichen Formel berechnet:\n");
  printf("          Real               Imag\n");

  if (fabsl(Hx1re) > (Hx2re))
  {
    printf("x1: % #LE       % #LE    \n",Hx1re,Hx1im);
    printf("x2: % #LE       % #LE\n",Hx2re,Hx2im);
  }
  else
  {
    printf("x1: % #LE       % #LE    \n",Hx2re,Hx2im);
    printf("x2: % #LE       % #LE\n",Hx1re,Hx1im);
  }
}



void Maschine()
{
long double Diskr,Faktor ;

  if (fabsl(p) > 1.0) {
    Faktor = fabsl(p);
    Diskr = 1.0/4.0 - q/p/p;
  }
  else {
    Faktor = 1.0;
    Diskr = p/2.0*p/2.0 - q;
  }
  if (Diskr < 0.0) {
    Mx1re = -p/2.0;
    Mx1im = Faktor * sqrtl(-Diskr);
    Mx2re = Mx1re;
    Mx2im = -Mx1im;
  }
  else
  {
    Mx1re = fabsl(p/2.0) + Faktor * sqrtl(Diskr);
    if (p > 0.0)  {Mx1re = -Mx1re;}
    if (sgn(Mx1re) == 0)  {Mx2re = 0.0;}
    else
    {Mx2re = q/Mx1re;}
  }
  printf("\nMit der maschinengerechten Formel berechnet:\n");
  printf("          Real              Imag    \n");
  printf("x1: % #LE       % #LE      \n",Mx1re,Mx1im);
  printf("x2: % #LE       % #LE\n",Mx2re,Mx2im);
}

// fuer die Rueckrechnung
struct komplex kadd(struct komplex z1, struct komplex z2)
{
  struct komplex z3;
  z3.re = z1.re + z2.re;
  z3.im = z1.im + z2.im;
  return z3;
}

struct komplex kmult(struct komplex z1, struct komplex z2)
{
  struct komplex z3;
  z3.re = (z1.re*z2.re) - (z1.im*z2.im);
  z3.im = (z1.re*z2.im) + (z1.im*z2.re);
  return z3;
}

struct komplex kneg(struct komplex z)
{
  struct komplex z3;
  z3.re = -z.re;
  z3.im = -z.im;
  return z3;
}

void RUeberschrift()
{
  printf("Diese quadratische Gleichung wurde eingegeben:\n");
  printf("X^2    +     p       *  x     +      q");
  printf("     = 0\n");
  printf("x^2    % #LE *  x   ",p);
  printf("% #LE  = 0\n\n",q);
  printf("\n\nJetzt werden die Koeffizienten p und q rueckgerechnet:\n");
}

void RHaendisch()
{
  struct komplex H1,H2,P,Q;
    H1.re = Hx1re; H1.im = Hx1im;  H2.re = Hx2re; H2.im = Hx2im;
    P = kadd(H1,H2);
    P = kneg(P);
    Hrpre = P.re; Hrpim = P.im;
    H1.re = Hx1re; H1.im = Hx1im;
    Q = kmult(H1,H2);
    Hrqre = Q.re; Hrqim = Q.im;
    printf("\n");
    printf("\n");
    printf("Uebliche Formel Ergebnisse zurueckgerechnet:\n");
    printf("         Real               Imag\n");
    printf("p:  % #LE       % #LE\n",Hrpre,Hrpim);
    printf("q:  % #LE       % #LE\n",Hrqre,Hrqim);
}

void RMaschine()
{
  struct komplex H1,H2,P,Q;

    H1.re = Mx1re; H1.im = Mx1im;  H2.re = Mx2re; H2.im = Mx2im;
    P = kadd(H1,H2);
    P = kneg(P);
    Mrpre = P.re; Mrpim = P.im;
    H1.re = Mx1re; H1.im = Mx1im;
    Q = kmult(H1,H2);
    Mrqre = Q.re; Mrqim = Q.im;
    printf("\n");
    printf("\n");
    printf("Maschinengerechte Formel Ergebnisse zurueckgerechnet:\n");
    printf("         Real               Imag\n");
    printf("p:  % #LE      % #LE\n",Mrpre,Mrpim);
    printf("q:  % #LE      % #LE\n",Mrqre,Mrqim);
}   